Imports Microsoft.Win32


Public Class frmMain

    Public arrIsoNone As String() = New String() {}
    Public arrIsoShared As String() = New String() {}
    Public arrIsoIsolated As String() = New String() {}
    Public str64bitDrivers As String = "SYSTEM\CurrentControlSet\Control\Print\Environments\Windows x64\Drivers\Version-3"
    Public str32bitDrivers As String = "SYSTEM\CurrentControlSet\Control\Print\Environments\Windows NT x86\Drivers\Version-3"
    Public oTable As New DataTable
    Public bActiveIsoDetection As Boolean = False
    Public ht32Drivers As New Hashtable
    Public ht64drivers As New Hashtable

    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim i32count As Integer = 0
        Dim i64count As Integer = 0

        setup()
       
        GetIsolationExplicit()

        LoadDrivers(str64bitDrivers)
        i64count = dgvMain.RowCount
        status("64-bit drivers: " & i64count & vbTab)

        LoadDrivers(str32bitDrivers)
        i32count = dgvMain.RowCount - i64count
        status("  32-bit drivers: " & i32count)
        dgvMain.Text = "rows " & oTable.Rows.Count
    End Sub

    Sub status(ByVal txt As String)
        lblStatus.Text &= txt
    End Sub

    Sub log(ByVal strmsg As String)
        Dim strlog As String = Application.StartupPath & "\log.txt"
        Dim owrite As IO.StreamWriter

        Try
            owrite = New IO.StreamWriter(strlog, True)
            owrite.WriteLine(Now & " - " & strmsg)
            owrite.Close()
            owrite.Dispose()
            owrite = Nothing
        Catch ex As Exception

        End Try
    End Sub

    Sub setup()
        Dim pInfo() As System.Reflection.PropertyInfo
        Dim Drv As New Driver

        'get column names
        pInfo = Drv.GetType.GetProperties
        For Each p As System.Reflection.PropertyInfo In pInfo
            If p.CanRead Then

                If p.Name.ToLower.IndexOf("date") <> -1 Then
                    Try
                        Dim t As System.Type = System.Type.GetType("System.DateTime")
                        oTable.Columns.Add(p.Name, t)
                    Catch ex As Exception
                        oTable.Columns.Add(p.Name)
                    End Try
                   
                Else
                    oTable.Columns.Add(p.Name)
                End If


                'If p.Name.ToLower.IndexOf("isolation") <> -1 Then
                '    oTable.Columns.Add("Isolation In Use")
                'End If
            End If
        Next
    End Sub


    Sub GetIsolationExplicit()
        Dim oReg As RegistryKey
        Dim iso As String = ""
        Dim L1 As String()
        Dim strPrintDelim As String = "\"

        oReg = Registry.LocalMachine.OpenSubKey("SYSTEM\CurrentControlSet\Control\Print")
        iso = oReg.GetValue("PrintDriverIsolationGroups", "")
        oReg.Close()




        If Trim(iso) <> "" Then

            log("double slashes found: " & (Split(iso, "\\").Length - 1))
            If Split(iso, "\\").Length > 3 Then
                log("found abnormal delims in reg. using loose mode!")
                log("refore replace: " & iso)
                iso = Replace(iso, "\\\\", "\\\\filler\\\\")
                iso = Replace(iso, "\\", "\")
                log("after replace: " & iso)
                L1 = Split(iso, "\\")
                strPrintDelim = "\"
            Else
                log("using normal delim mode")
                L1 = Split(iso, "\\")
                strPrintDelim = "\"
            End If



            If L1.Length = 3 Then
                log("found 3 isolation modes in reg")
                bActiveIsoDetection = True

                arrIsoNone = Split(L1(0), strPrintDelim)
                log("isolation none: " & arrIsoNone.Length)

                arrIsoShared = Split(L1(1), strPrintDelim)
                log("isolation shared: " & arrIsoShared.Length)

                arrIsoIsolated = Split(L1(2), strPrintDelim)
                log("isolation isolated: " & arrIsoIsolated.Length)
            Else
                log("primary isolation modes found in registry: " & L1.Length)
                log("debug: regkey - " & iso)
            End If
        End If
       



    End Sub

    Function GetCurrentIsoMode(ByVal drv As Driver) As String
        Dim strDrvName As String = drv.DriverName


        For Each item As String In arrIsoNone
            If item.ToLower.IndexOf(strDrvName.ToLower) <> -1 Then
                Return "None - By Admin"
            End If
        Next


        For Each item As String In arrIsoShared
            If item.ToLower.IndexOf(strDrvName.ToLower) <> -1 Then
                Return "Shared - By Admin"
            End If
        Next

        For Each item As String In arrIsoIsolated
            If item.ToLower.IndexOf(strDrvName.ToLower) <> -1 Then
                Return "Isolated - By Admin"
            End If
        Next

        If bActiveIsoDetection = True Then
            If drv.IsolationSupport.ToLower = "yes" Then
                Return "Shared - System Default"
            Else
                Return "None - System Default"
            End If

        Else
            Return "Detection Failed Check Registry"
        End If

    End Function

    Sub LoadDrivers(ByVal strKey As String) ' As Driver()
        Dim oReg As RegistryKey
        Dim Drv As Driver

        oReg = Registry.LocalMachine.OpenSubKey(strKey)

        If oReg.GetSubKeyNames.Length > 0 Then
            Dim i As Integer = 0
            For Each s As String In oReg.GetSubKeyNames
                'ReDim Preserve Drv(i)
                LoadDrivers(strKey & "\" & s)
                i += 1
            Next
        ElseIf oReg.GetValue("Version", -1) <> -1 Then
            Drv = ReadDriver(strKey)
            'Drv.GetIsolationSupportFromInf()



            'mismatch driver checking
            DriverHealthCheck(Drv)

            Try
                'variables
                Dim pInfo() As System.Reflection.PropertyInfo
                Dim oRow As DataRow

                'load one row
                oRow = oTable.NewRow
                pInfo = Drv.GetType.GetProperties
                For Each p As System.Reflection.PropertyInfo In pInfo
                    If p.CanRead Then
                        oRow(p.Name) = CallByName(Drv, p.Name, CallType.Get)
                    End If
                Next

              

                oTable.Rows.Add(oRow)

                dgvMain.DataSource = oTable

                Application.DoEvents()


            Catch ex As Exception
                MsgBox(ex.Message)
            End Try
        End If


    End Sub

    Sub DriverHealthCheck(ByRef drv As Driver)

        If drv.ProcArch.IndexOf("32") <> -1 Then

            If ht64drivers.ContainsKey(drv.DriverName) Then
                Dim o64ver As Driver = CType(ht64drivers(drv.DriverName), Driver)
                If o64ver.DriverVersion <> drv.DriverVersion Then
                    drv.errormsg = "32 ver. not match 64 ver."
                End If
            End If

        ElseIf drv.ProcArch.IndexOf("64") <> -1 Then

            If ht32Drivers.ContainsKey(drv.DriverName) Then
                Dim o32ver As Driver = CType(ht32Drivers(drv.DriverName), Driver)

                If o32ver.DriverVersion <> drv.DriverVersion Then
                    drv.errormsg = "64 ver. not match 32 ver."
                End If
            End If

        End If
    End Sub

    Function ReadDriver(ByVal strkey As String) As Driver
        Dim oReg As RegistryKey
        Dim drv As New Driver

        oReg = Registry.LocalMachine.OpenSubKey(strkey)
        drv.DriverName = GetLastName(strkey)

        drv.ConfigFile = oReg.GetValue("Configuration File", "-1")
        drv.URL = oReg.GetValue("OEM URL", "-1")
        drv.PrintProcessor = oReg.GetValue("Print Processor", "-1")
        drv.infpath = oReg.GetValue("InfPath", "-1")
        drv.DriverVersion = oReg.GetValue("DriverVersion", "-1")
        drv.WinDrvVersion = oReg.GetValue("Version", "-1")
        drv.DataFile = oReg.GetValue("Data File", "-1")
        drv.DriverDate = oReg.GetValue("DriverDate", "-1")

        drv.ProcArch = DriverArch(strkey)

        If drv.ProcArch.IndexOf("32") <> -1 Then
            ht32Drivers(drv.DriverName) = drv
        ElseIf drv.ProcArch.IndexOf("64") <> -1 Then
            ht64drivers(drv.DriverName) = drv
        End If

        drv.GetIsolationSupportFromInf()

        drv.IsolationInUse = GetCurrentIsoMode(drv)

        oReg.Close()

        Return drv
    End Function

    Function DriverArch(ByVal strkey As String) As String
        If strkey.ToLower.IndexOf("x86") <> -1 Then
            Return ("32-BIT")
        ElseIf strkey.ToLower.IndexOf("x64") <> -1 Then
            Return ("64-BIT")
        Else
            Return "unknown"
        End If
    End Function

    Function GetLastName(ByVal strval As String) As String
        Dim arrVals As String() = strval.Split("\")

        Dim strret As String = arrVals(arrVals.Length - 1)

        If strret = "" Then
            strret = arrVals(arrVals.Length - 2)
        End If

        Return strret

    End Function

    Private Sub dgvMain_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles dgvMain.DoubleClick

        Try

            Dim f As String = dgvMain.Item("InfPath", dgvMain.CurrentRow.Index).Value
            If IO.File.Exists(f) Then
                System.Diagnostics.Process.Start(f)

            End If

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    Private Sub btnLog_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLog.Click
        Try
            System.Diagnostics.Process.Start(Application.StartupPath & "\log.txt")
        Catch ex As Exception

        End Try
    End Sub

    Private Sub txtbxFilter_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtbxFilter.TextChanged
        For Each r As DataGridViewRow In dgvMain.Rows
            Try

                If dgvMain.Item("DriverName", r.Index).Value.ToString.ToLower.IndexOf(Trim(txtbxFilter.Text.ToLower)) <> -1 Then
                    r.Visible = True
                Else
                    If r.Displayed = True Then
                        ' dgvMain.CurrentCell = Nothing
                        r.Visible = False
                    End If

                End If
            Catch ex As Exception

            End Try
        Next
    End Sub
End Class



Public Class Driver
    Private _DriverVersion As String
    Private _DriverDate As String
    Private _infpath As String
    Private _DriverName As String
    Private _ProcArch As String
    Private _WinDrvVersion As String
    Private _DataFile As String
    Private _ConfigFile As String
    Private _PrintProcessor As String
    Private _IsolationSupport As String
    Private _IsolationInUse As String
    Private _URL As String
    Private _ErrorMsg As String


    Public Property DriverVersion() As String
        Get
            Return _DriverVersion
        End Get
        Set(ByVal value As String)
            _DriverVersion = value
        End Set
    End Property

    Public Property DriverDate() As String
        Get
            Return _DriverDate
        End Get
        Set(ByVal value As String)
            _DriverDate = value
        End Set
    End Property

    Public Property URL() As String
        Get
            Return _URL
        End Get
        Set(ByVal value As String)
            _URL = value
        End Set
    End Property

    Public Property infpath() As String
        Get
            Return _infpath
        End Get
        Set(ByVal value As String)
            _infpath = value
            GetIsolationSupportFromInf()
        End Set
    End Property

    Public Property DriverName() As String
        Get
            Return _DriverName
        End Get
        Set(ByVal value As String)
            _DriverName = value
        End Set
    End Property

    Public Property ProcArch() As String
        Get
            Return _ProcArch
        End Get
        Set(ByVal value As String)
            _ProcArch = value
        End Set
    End Property

    Public Property WinDrvVersion() As String
        Get
            Return _WinDrvVersion
        End Get
        Set(ByVal value As String)
            _WinDrvVersion = value
        End Set
    End Property

    Public Property DataFile() As String
        Get
            Return _DataFile
        End Get
        Set(ByVal value As String)
            _DataFile = value
        End Set
    End Property

    Public Property ConfigFile() As String
        Get
            Return _ConfigFile
        End Get
        Set(ByVal value As String)
            _ConfigFile = value
        End Set
    End Property

    Public Property PrintProcessor() As String
        Get
            Return _PrintProcessor
        End Get
        Set(ByVal value As String)
            _PrintProcessor = value
        End Set
    End Property

    Public Property IsolationSupport() As String
        Get
            Return _IsolationSupport
        End Get
        Set(ByVal value As String)
            _IsolationSupport = value
        End Set
    End Property

    Public Property IsolationInUse() As String
        Get
            Return _IsolationInUse
        End Get
        Set(ByVal value As String)
            _IsolationInUse = value
        End Set
    End Property

    Public Property errormsg() As String
        Get
            Return _errormsg
        End Get
        Set(ByVal value As String)
            _errormsg = value
        End Set
    End Property

    Public Sub GetIsolationSupportFromInf()
        Dim oRead As IO.StreamReader
        If Trim(infpath) = "" Then
            errormsg = "NO INF Key"
            _IsolationSupport = "Unknown"
            Exit Sub
        ElseIf IO.File.Exists(infpath) = False Then
            errormsg = "Inf Missing From Disk"
            _IsolationSupport = "Unknown"
            Exit Sub
        End If


        'inf file exists
        Try
            _IsolationSupport = "Not Found in INF"

            oRead = New IO.StreamReader(_infpath)
            While oRead.Peek <> -1
                Dim line As String = oRead.ReadLine.ToLower

                If line.IndexOf("driverisolation") <> -1 Then
                    If line.IndexOf("2") <> -1 Then
                        _IsolationSupport = "YES"
                        frmMain.log("driver: " & _DriverName & ", isolation: 2")
                        Exit While
                    ElseIf line.IndexOf("0") <> -1 Then
                        _IsolationSupport = "No"
                        frmMain.log("driver: " & _DriverName & ", isolation: 0")
                        Exit While
                    End If
                End If
            End While

            frmMain.log("driver: " & _DriverName & ", isolation: unknown")


            oRead.Close()
            oRead.Dispose()
            oRead = Nothing
        Catch ex As Exception
            errormsg = "err: " & ex.Message
            Exit Sub
        End Try
    End Sub
End Class
